package tantra;

import java.io.*;
import java.util.*;
import tantra.DateEx.*;

/**
 * 過去のhtmlファイルなどを誤って操作しないように、ファイルをロックできるようにするためのクラス。
 * 
 * @author Luft
 * 
 */
public class FixedFileList {

	/** ロックされたファイルのリストが保存されたファイルの名前 */
	private final static String FIXED_FILE_LIST_NAME = "fixed_file_list.txt";
	/** ロックされたファイルのリスト */
	private Vector<String> fixed_file_list = new Vector<String>();


	/**
	 * 開こうとしているファイルがfixされているかどうかを判定します
	 * 
	 * @param fpath
	 *        判定対象のファイルの名前
	 * @return <code>fpath</code>がfixされていればtrueを、そうでなければfalseを返します。
	 */
	public boolean isFixed( String fpath ){
		if( fixed_file_list.size() <= 0 ){
			return false;
		}else{
			int i;
			for( i = 0; i <= fixed_file_list.size() - 1; i++ ){
				if( fpath.equals( fixed_file_list.elementAt( i ) ) ){
					return true;
				}
			}
			return false;
		}
	}


	/**
	 * fixed_file_list.txtを読み込みます
	 */
	public void read(){
		String fpath = tantra.settings.total_data_location + File.separator
				+ FIXED_FILE_LIST_NAME;

		if( !tantra.exists( fpath ) ){
			return;
		}

		// 行数を調べる
		fixed_file_list.removeAllElements();
		try{
			BufferedReader sr = new BufferedReader( new InputStreamReader(
					new FileInputStream( fpath ), tantra.ShiftJIS ) );
			String line;
			while( (line = sr.readLine()) != null ){
				fixed_file_list.add( line );
			}
			sr.close();

		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * fnameで指定したファイルをFIX指定にします
	 * 
	 * @param fname
	 *        FIX指定にするファイルの名前
	 * @return FIXに成功すればtrueを、そうでなければfalseを返します。
	 */
	private boolean fix( String fname ){
		if( fixed_file_list.size() <= 0 ){
			fixed_file_list.add( fname );
			return true;
		}else{
			if( !fixed_file_list.contains( fname ) ){
				fixed_file_list.add( fname );
				return true;
			}else{
				return false;
			}
		}
	}


	/**
	 * ファイルをfix指定にします。日付<code>d</code>分のhtmlファイルどもを纏めてfixします。
	 * 
	 * @param d
	 *        fixするファイルの日付。
	 */
	public void fixFiles( DateEx d ){
		if( d.isNull() ) return;
		String ymd = d.toString( Format.yyyyMMdd );
		int added;
		added = 0;
		if( fix( tantra.settings.target_kruma + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "lord_" + tantra.settings.target_kruma + "_"
				+ Lord.Brahma.toStringEn() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "lord_" + tantra.settings.target_kruma + "_"
				+ Lord.Vishnu.toStringEn() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "lord_" + tantra.settings.target_kruma + "_"
				+ Lord.Shiva.toStringEn() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "running_info_" + tantra.settings.target_kruma + "_" + ymd
				+ ".html" ) ) added += 1;
		if( fix( "population_" + tantra.settings.target_kruma + "_" + ymd
				+ ".html" ) ) added += 1;
		if( fix( "picture_" + tantra.settings.target_kruma + "_"
				+ d.addDays( 0 ).toString( Format.yyyyMMdd ) + ".png" ) ) added += 1;
		if( fix( "picture_" + tantra.settings.target_kruma + "_"
				+ d.addDays( 1 ).toString( Format.yyyyMMdd ) + ".png" ) ) added += 1;
		if( fix( "picture_" + tantra.settings.target_kruma + "_"
				+ d.addDays( 2 ).toString( Format.yyyyMMdd ) + ".png" ) ) added += 1;
		if( fix( "picture_" + tantra.settings.target_kruma + "_"
				+ d.addDays( 3 ).toString( Format.yyyyMMdd ) + ".png" ) ) added += 1;
		if( fix( "picture_" + tantra.settings.target_kruma + "_"
				+ d.addDays( 4 ).toString( Format.yyyyMMdd ) + ".png" ) ) added += 1;
		if( fix( "picture_" + tantra.settings.target_kruma + "_"
				+ d.addDays( 5 ).toString( Format.yyyyMMdd ) + ".png" ) ) added += 1;
		if( fix( "picture_" + tantra.settings.target_kruma + "_"
				+ d.addDays( 6 ).toString( Format.yyyyMMdd ) + ".png" ) ) added += 1;
		if( fix( "doctor_" + tantra.settings.target_kruma + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "doctor_" + tantra.settings.target_kruma + "_"
				+ Lord.Brahma.toStringEn() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "doctor_" + tantra.settings.target_kruma + "_"
				+ Lord.Vishnu.toStringEn() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "doctor_" + tantra.settings.target_kruma + "_"
				+ Lord.Shiva.toStringEn() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "job_" + tantra.settings.target_kruma + "_"
				+ Job.Asura_Rakshasa.toString() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "job_" + tantra.settings.target_kruma + "_"
				+ Job.Deva_Garuda.toString() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "job_" + tantra.settings.target_kruma + "_"
				+ Job.Naga_Kimnara.toString() + "_" + ymd + ".html" ) ) added += 1;
		if( fix( "job_" + tantra.settings.target_kruma + "_"
				+ Job.Yaksa_Gandharva.toString() + "_" + ymd + ".html" ) ) added += 1;

		// ファイルに保存
		String fpath;
		fpath = tantra.settings.total_data_location + File.separator
				+ FIXED_FILE_LIST_NAME;
		try{
			BufferedWriter unit = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( fpath ), tantra.ShiftJIS ) );
			int i;
			String file;
			for( i = 0; i <= fixed_file_list.size() - 1; i++ ){
				file = (String) fixed_file_list.elementAt( i );
				unit.write( file );
				unit.newLine();
			}
			unit.close();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}

}
