package tantra;

import java.io.*;
import java.text.DecimalFormat;
import java.util.Vector;
import tantra.DateEx.*;

/**
 * HTMLファイルへの出力などを行う。
 * 
 * @author Luft
 */
public class Html {

	/** &amp;nbsp;&amp;nbsp;&amp;gt;&amp;gt;&amp;nbsp;&amp;nbsp; */
	public static final String GT = "&nbsp;&nbsp;&gt;&gt;&nbsp;&nbsp;";
	/** &lt;a href="index.html"&gt;top&lt;/a&gt; */
	public static final String LINK_TOP = "<a href=\"index.html\">top</a>";


	/**
	 * アフィリエイト用のタグを書き込みます．
	 * タグは，settings.total_data_locationのaffiliate.txtファイルに用意しておきます
	 * 
	 * @param unit
	 *        タグを書き込む対象となるファイルのBufferedWriter
	 */
	public static void addAffiliate( BufferedWriter unit ){
		String fpath = tantra.settings.total_data_location + File.separator
				+ tantra.AFFILIATE_FILE_NAME;
		if( tantra.exists( fpath ) ){
			try{
				BufferedReader unit2 = new BufferedReader(
						new InputStreamReader( new FileInputStream( fpath ),
								tantra.ShiftJIS ) );
				String line;
				while( (line = unit2.readLine()) != null ){
					unit.write( line );
					unit.newLine();
				}
				unit2.close();
			}catch( Exception e ){
				tantra.printError( e );
			}
		}
	}


	/**
	 * ファイルストリームに，HTMLファイルヘッダーを書き込みます
	 * 
	 * @param unit
	 *        書き込み対象のBufferedWriter
	 * @param title
	 *        &lt;title&gt;タグで指定するHTMLのタイトル
	 */
	public static void writeHeader( BufferedWriter unit, String title ){
		try{
			unit
					.write( "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">" );
			unit.newLine();
			unit.write( "<html lang=\"ja\">" );
			unit.newLine();
			unit.write( "<head>" );
			unit.newLine();
			// Dim ver As System.Diagnostics.FileVersionInfo = _
			// System.Diagnostics.FileVersionInfo.GetVersionInfo( _
			// System.Reflection.Assembly.GetExecutingAssembly().Location)
			unit.write( "<meta name=\"generator\" content=\"tantra_data "
					+ tantra.ver + "\">" );
			unit.newLine();
			unit
					.write( "<meta http-equiv=Content-Type content=\"text/html; charset=SHIFT_JIS\">" );
			unit.newLine();
			unit
					.write( "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">" );
			unit.newLine();
			unit
					.write( "<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\">" );
			unit.newLine();
			unit.write( "<title>" + title + " [generator version " + tantra.ver
					+ "]</title>" );
			unit.newLine();
			unit
					.write( "<link href=\"style.css\" rel=\"stylesheet\" type=\"text/css\">" );
			unit.newLine();
			unit.write( "</head>" );
			unit.newLine();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * アンカータグを付加します。
	 * 
	 * @param href
	 *        アンカー先のURI。ex."index.html"
	 * @param name
	 *        HTMLに表示される文字列。ex."index"
	 * @return アンカータグを付加した文字列。&lt;a href="index.html"&gt;index&lt;/a&gt;
	 */
	public static String tagA( String href, String name ){
		return tagA( href, name, "" );
	}


	/**
	 * アンカータグを付加します。アンカー先のファイルが実際に存在するかどうかをチェックします
	 * 
	 * @param href
	 *        アンカー先のURI。ex."index.html"
	 * @param name
	 *        HTMLに表示される文字列。ex."index"
	 * @param path
	 *        アンカー先のファイル。".\index.html"
	 * @return pathが存在していれば、「&lt;a href="index.html"&gt;index&lt;/a&gt;」。
	 *         存在していなければ、「index」を返す。
	 */
	public static String tagA( String href, String name, String path ){
		if( path.equals( "" ) ){
			path = tantra.settings.html_location;
		}
		File fi = new File( path + File.separator + href );
		if( fi.exists() ){
			return "<a href=\"" + href + "\">" + name + "</a>";
		}else{
			return name;
		}
	}


	/**
	 * インデックスを更新します
	 */
	public static void createIndex(){
		String fpath = tantra.settings.html_location;

		// htmlファイルをリストアップ
		File dir = new File( fpath );
		String filelist[];
		filelist = dir.list( new FilterHtml() );
		Vector<String> vfile = new Vector<String>();

		int i;
		for( i = 0; i < filelist.length; i++ ){
			if( filelist[i].startsWith( tantra.settings.target_kruma ) ){
				filelist[i] = filelist[i].replace( tantra.settings.target_kruma
						+ "_", "" );
				filelist[i] = filelist[i].replace( ".html", "" );
				vfile.add( filelist[i] );
			}
			// System.out.println( "Html::updateIndexHtml;i,filelist[i]=" + i +
			// "," + filelist[i] );
		}

		// 新しい順に並べ替え
		int d1, d2;
		int change;
		String tmpstr;
		if( vfile.size() <= 0 ){
			return;
		}
		change = 1;
		while( change > 0 ){
			change = 0;
			for( i = 0; i < vfile.size() - 1; i++ ){
				d1 = Integer.valueOf( vfile.elementAt( i ) );
				d2 = Integer.valueOf( vfile.elementAt( i + 1 ) );
				if( d2 > d1 ){
					tmpstr = vfile.elementAt( i );
					vfile.setElementAt( vfile.elementAt( i + 1 ), i );
					vfile.setElementAt( tmpstr, i + 1 );
					change += 1;
				}
			}
		}

		// ファイル出力
		try{
			BufferedWriter sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( tantra.settings.html_location
							+ File.separator + "index_"
							+ tantra.settings.target_kruma + ".html" ),
					tantra.ShiftJIS ) );
			writeHeader( sw, "Ranking List" );
			sw.write( "<body>" );
			sw.newLine();
			int y, m, d;
			DateEx tmpd;
			// String result;
			String navigation;
			navigation = "<p>" + "<a href=\"index.html\">top</a>" + GT
					+ tantra.settings.target_kruma + "</p>";
			sw.write( navigation );
			sw.newLine();
			sw.write( "<h4>weekly ranking</h4>" );
			sw.newLine();
			sw.write( "<div class=\"indent\">" );
			sw.newLine();
			sw
					.write( "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" width=\"100%\">" );
			sw.newLine();
			sw.write( "<tr>" );
			sw.newLine();
			sw.write( "  <td rowspan=\"2\" class=title>集計期間</td>" );
			sw.newLine();
			sw.write( "  <td colspan=\"2\" class=title>総合値順ランキング</td>" );
			sw.newLine();
			sw.write( "  <td colspan=\"2\" class=title>薬値順ランキング</td>" );
			sw.newLine();
			sw.write( "  <td rowspan=\"2\" class=title>種族別ランキング</td>" );
			sw.newLine();
			sw.write( "  <td rowspan=\"2\" class=title>欠測時間<br>等の情報</td>" );
			sw.newLine();
			sw.write( "  <td rowspan=\"2\" class=title>聖地人口の<br>時間推移</td>" );
			sw.newLine();
			sw.write( "</tr>" );
			sw.newLine();
			sw.write( "<tr>" );
			sw.newLine();
			sw.write( "  <td class=title>全体</td>" );
			sw.newLine();
			sw.write( "  <td class=title>主神別</td>" );
			sw.newLine();
			sw.write( "  <td class=title>全体</td>" );
			sw.newLine();
			sw.write( "  <td class=title>主神別</td>" );
			sw.newLine();
			sw.write( "</tr>" );
			sw.newLine();
			// DateEx tmpd_end;
			for( i = 0; i <= vfile.size() - 1; i++ ){
				y = Integer.valueOf( vfile.elementAt( i ).substring( 0, 4 ) );
				m = Integer.valueOf( vfile.elementAt( i ).substring( 4, 6 ) );
				d = Integer.valueOf( vfile.elementAt( i ).substring( 6, 8 ) );
				if( y >= 2000 ){
					sw.write( "<tr>" );
					sw.newLine();
					tmpd = new DateEx( y, m, d, 0, 0, 0 );
					String date = tmpd.toString( Format.yyyyMMdd );
					// tmpd_end = tmpd.addDays( 6 );
					sw.write( "  <td class=n>"
							+ tmpd.toString( Format.yyyy_MM_dd ) + " -</td>" );
					sw.newLine();

					sw.write( "  <td class=c>"
							+ tagA( tantra.settings.target_kruma + "_"
									+ vfile.elementAt( i ) + ".html", "全体" )
							+ "</td>" );
					sw.newLine();
					sw.write( "  <td class=c>" );
					sw.write( tagA( "lord_" + tantra.settings.target_kruma
							+ "_" + Lord.Brahma.toStringEn() + "_" + date
							+ ".html", "黄" )
							+ "&nbsp;&nbsp;&nbsp;" );
					sw.write( tagA( "lord_" + tantra.settings.target_kruma
							+ "_" + Lord.Vishnu.toStringEn() + "_" + date
							+ ".html", "赤" )
							+ "&nbsp;&nbsp;&nbsp;" );
					sw.write( tagA( "lord_" + tantra.settings.target_kruma
							+ "_" + Lord.Shiva.toStringEn() + "_" + date
							+ ".html", "青" )
							+ "</td>" );
					sw.newLine();

					sw.write( "  <td class=c>"
							+ tagA( "doctor_" + tantra.settings.target_kruma
									+ "_" + vfile.elementAt( i ) + ".html",
									"全体" ) + "</td>" );
					sw.newLine();
					sw.write( "  <td class=c>" );
					sw.write( tagA( "doctor_" + tantra.settings.target_kruma
							+ "_" + Lord.Brahma.toStringEn() + "_" + date
							+ ".html", "黄" )
							+ "&nbsp;&nbsp;&nbsp;" );
					sw.write( tagA( "doctor_" + tantra.settings.target_kruma
							+ "_" + Lord.Vishnu.toStringEn() + "_" + date
							+ ".html", "赤" )
							+ "&nbsp;&nbsp;&nbsp;" );
					sw.write( tagA( "doctor_" + tantra.settings.target_kruma
							+ "_" + Lord.Shiva.toStringEn() + "_" + date
							+ ".html", "青" )
							+ "</td>" );
					sw.newLine();

					sw.write( "  <td class=c>"
							+ tagA( "job_" + tantra.settings.target_kruma + "_"
									+ Job.Asura_Rakshasa.toString() + "_"
									+ date + ".html", "AR" )
							+ "&nbsp;&nbsp;&nbsp;" );
					sw.write( tagA(
							"job_" + tantra.settings.target_kruma + "_"
									+ Job.Deva_Garuda.toString() + "_" + date
									+ ".html", "DG" )
							+ "&nbsp;&nbsp;&nbsp;" );
					sw.write( tagA( "job_" + tantra.settings.target_kruma + "_"
							+ Job.Naga_Kimnara.toString() + "_" + date
							+ ".html", "NK" )
							+ "&nbsp;&nbsp;&nbsp;" );
					sw.write( tagA( "job_" + tantra.settings.target_kruma + "_"
							+ Job.Yaksa_Gandharva.toString() + "_" + date
							+ ".html", "YG" )
							+ "</td>" );
					sw.newLine();

					sw.write( "  <td class=c>"
							+ tagA( "running_info_"
									+ tantra.settings.target_kruma + "_" + date
									+ ".html", "詳細" ) + "</td>" );
					sw.newLine();

					sw.write( "  <td class=c>"
							+ tagA( "population_"
									+ tantra.settings.target_kruma + "_" + date
									+ ".html", "詳細" ) + "</td>" );
					sw.newLine();
					sw.write( "</tr>" );
					sw.newLine();
				}
			}
			sw.write( "</table>" );
			sw.newLine();
			sw.write( "</div>" );
			sw.newLine();
			sw.write( "<hr>" );
			sw.newLine();
			sw.write( navigation );
			sw.newLine();
			addAffiliate( sw );
			sw.write( "</body></html>" );
			sw.newLine();
			sw.close();
			tantra.uploader_main.registerUpload( tantra.settings.html_location
					+ File.separator + "index_" + tantra.settings.target_kruma
					+ ".html" );
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * ウィークリー聖地人勝率順リストをhtmlに書き込みます
	 * 
	 * @param week_start
	 */
	public static void createRanking( DateEx iweek_start ){
		String fname;
		Vector<PersonalData> pd = new Vector<PersonalData>();
		int i, index;
		Vector<PersonalData> pd_lord = new Vector<PersonalData>();
		final int LIST_BORDER = 10;
		DateEx week_start = iweek_start.weekStart();
		/*
		 * System.out .println(
		 * "Html::makeHtmlKrumaWeekly;tantra.kruma_weekly.size()=" +
		 * tantra.kruma_weekly.size() );
		 */
		if( tantra.kruma_weekly.size() <= 0 ){
			return;
		}

		// キルログの合計数が少ない人、ランキングからの除外希望者を除外
		int j;
		for( i = 0; i < tantra.kruma_weekly.size(); i++ ){
			if( tantra.kruma_weekly.elementAt( i ).win
					+ tantra.kruma_weekly.elementAt( i ).lose > LIST_BORDER
					&& !tantra.suspects.isSuspect( tantra.kruma_weekly
							.elementAt( i ).name ) ){
				index = PersonalData.indexOf( pd, tantra.kruma_weekly
						.elementAt( i ).name, true );
				pd.setElementAt( tantra.kruma_weekly.elementAt( i ), index );
				j = PersonalData.indexOf( tantra.kruma,
						pd.elementAt( index ).name, false );
				if( j >= 0 ){
					// PersonalData tmppd = pd.elementAt( index );
					// PersonalData pd_kruma = tantra.kruma.elementAt( j );
					pd.elementAt( index ).brahma = tantra.kruma.elementAt( j ).brahma;
					pd.elementAt( index ).vishnu = tantra.kruma.elementAt( j ).vishnu;
					pd.elementAt( index ).shiva = tantra.kruma.elementAt( j ).shiva;
					//pd.elementAt( index ).job = tantra.kruma.elementAt( j ).job;
					// tmppd.brahma = pd_kruma.brahma;
					// tmppd.vishnu = pd_kruma.vishnu;
					// tmppd.brahma = pd_kruma.shiva;
					// tmppd.job = pd_kruma.job;
					// pd.setElementAt( tmppd, index );
				}
			}
		}

		/*
		 * 総合値・薬値順
		 */
		// 全体
		//System.out.println( "createRanking;All-total;pd.size()=" + pd.size() );
		fname = tantra.settings.target_kruma + "_"
				+ week_start.toString( Format.yyyyMMdd ) + ".html";
		makeHtmlKrumaWeeklyOutput( pd, fname, week_start,
				tantra.OutputMode.TOTAL, Lord.Unknown, Job.Unknown );
		fname = "doctor_" + tantra.settings.target_kruma + "_"
				+ week_start.toString( Format.yyyyMMdd ) + ".html";
		//System.out.println( "createRanking;All-doctor;pd.size()=" + pd.size() );
		makeHtmlKrumaWeeklyOutput( pd, fname, week_start,
				tantra.OutputMode.DOCTOR, Lord.Unknown, Job.Unknown );

		// Lord毎
		if( pd.size() <= 0 ) return;
		for( Lord target_lord : Lord.values() ){
			if( target_lord == Lord.Unknown ) break;
			pd_lord.removeAllElements();
			for( i = 0; i <= pd.size() - 1; i++ ){
				if( pd.elementAt( i ).detectLord() == target_lord ){
					j = PersonalData.indexOf( pd_lord, pd.elementAt( i ).name,
							true );
					if( j >= 0 ){
						pd_lord.setElementAt( pd.elementAt( i ), j );
					}
				}
			}
			fname = "lord_" + tantra.settings.target_kruma + "_"
					+ target_lord.toStringEn() + "_"
					+ week_start.toString( Format.yyyyMMdd ) + ".html";
			//System.out.println( "createRanking;lord-total;target_lord,pd_lord.size()=" + target_lord.toStringEn() + "," + pd_lord.size() );
			makeHtmlKrumaWeeklyOutput( pd_lord, fname, week_start,
					tantra.OutputMode.TOTAL, target_lord, Job.Unknown );
			fname = "doctor_" + tantra.settings.target_kruma + "_"
					+ target_lord.toStringEn() + "_"
					+ week_start.toString( Format.yyyyMMdd ) + ".html";
			//System.out.println( "createRanking;lord-doctor;target_lord,pd_lord.size()=" + target_lord.toStringEn() + "," + pd_lord.size() );
			makeHtmlKrumaWeeklyOutput( pd_lord, fname, week_start,
					tantra.OutputMode.DOCTOR, target_lord, Job.Unknown );
		}

		/*
		 * 種族毎
		 */
		// 種族別のリストを抽出しながら出力
		if( pd.size() <= 0 ) return;
		for( Job target_job : Job.values() ){
			if( target_job == Job.Unknown ) break;
			pd_lord.removeAllElements();
			for( i = 0; i <= pd.size() - 1; i++ ){
				if( tantra.joblist.getJob( pd.elementAt( i ).name ) == target_job ){
					j = PersonalData.indexOf( pd_lord, pd.elementAt( i ).name,
							true );
					if( j >= 0 ){
						pd_lord.setElementAt( pd.elementAt( i ), j );
					}
				}
			}
			fname = "job_" + tantra.settings.target_kruma + "_"
					+ target_job.toString() + "_"
					+ week_start.toString( Format.yyyyMMdd ) + ".html";
			//System.out.println( "createRanking;tribe;target_job,pd_lord.size()=" + target_job.toString() + "," + pd_lord.size() );
			makeHtmlKrumaWeeklyOutput( pd_lord, fname, week_start,
					tantra.OutputMode.JOB, Lord.Unknown, target_job );
		}
	}


	/**
	 * <code>pd()</code>で与えられた聖地人リストを，HTMLに書き出します
	 * 
	 * @param pd
	 *        出力する聖地人リスト
	 * @param html_name
	 *        出力するファイルへのパス
	 * @param week_start
	 *        出力する週の日曜日の日付
	 * @param mode
	 *        出力するモード
	 * @param target_lord
	 *        出力対象のLord。全Lordのファイルを出力したい場合はLord.Unknownを指定
	 * @param target_job
	 *        出力対象のJob。全Jobのファイルを出力したい場合はJob.Unknownを指定
	 */
	private static void makeHtmlKrumaWeeklyOutput( Vector<PersonalData> pd,
			String html_name, DateEx week_start, tantra.OutputMode mode,
			Lord target_lord, Job target_job ){

		if( !tantra.fixed.isFixed( tantra.settings.html_location
				+ File.separator + html_name ) ){
			try{
				BufferedWriter sw = new BufferedWriter(
						new OutputStreamWriter( new FileOutputStream(
								tantra.settings.html_location + File.separator
										+ html_name ), tantra.ShiftJIS ) );
				writeHeader( sw, week_start.toString( Format.yyyy_MM_dd ) );
				sw.write( "<body>" );
				sw.newLine();
				DateEx d = new DateEx();
				d.now();
				String navigation_bar;
				navigation_bar = "<p>"
						+ LINK_TOP
						+ GT
						+ tagA( "index_" + tantra.settings.target_kruma
								+ ".html", tantra.settings.target_kruma ) + GT
						+ week_start.toString( Format.yyyy_MM_dd ) + GT;
				switch( mode ){
					case TOTAL:
						navigation_bar = navigation_bar + "Total" + GT;
						break;
					case DOCTOR:
						navigation_bar = navigation_bar + "Doctor" + GT;
						break;
					case JOB:
						navigation_bar = navigation_bar + "Job" + GT;
						break;
				}
				if( mode == tantra.OutputMode.JOB ){
					navigation_bar = navigation_bar + target_job.toString()
							+ "</p>";
				}else{
					navigation_bar = navigation_bar + target_lord.toStringEn()
							+ "</p>";
				}
				sw.write( navigation_bar );
				sw.newLine();
				sw.write( "<p style=\"text-align:right;\">ログ作成日："
						+ d.toString( Format.yyyy_MM_dd ) + " "
						+ d.toString( Format.hh_mm_ss ) + "</p>" );
				sw.newLine();
				sw
						.write( "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" width = \"100%\">" );
				sw.newLine();
				sw.write( "<tr>" );
				sw.newLine();
				sw.write( "  <td class=title>Rank</td>" );
				sw.newLine();
				sw.write( "  <td class=title>Character Name</td>" );
				sw.newLine();
				sw.write( "  <td class=title>God</td>" );
				sw.newLine();
				sw.write( "  <td class=title>Tribe</td>" );
				sw.newLine();
				if( target_lord != Lord.Unknown ){
					sw.write( "  <td class=title>全体順位</td>" );
					sw.newLine();
				}
				sw.write( "  <td class=title>勝利回数</td>" );
				sw.newLine();
				sw.write( "  <td class=title>負け回数</td>" );
				sw.newLine();
				sw.write( "  <td class=title>総合値</td>" );
				sw.newLine();
				sw.write( "  <td class=title>活動時間</td>" );
				sw.newLine();
				sw.write( "  <td class=title>lose/hour</td>" );
				sw.newLine();
				sw.write( "  <td class=title>DMG値</td>" );
				sw.newLine();
				sw.write( "  <td class=title>薬値</td>" );
				sw.newLine();
				sw.write( "</tr>" );
				sw.newLine();

				// 並べ替え
				//System.out.println( "makeHtmlKrumaWeeklyOutput;pd.size()=" + pd.size() );
				if( pd.size() >= 1 ){
					double rank_ptr[] = new double[pd.size()];
					int i;
					for( i = 0; i <= pd.size() - 1; i++ ){
						rank_ptr[i] = pd.elementAt( i ).ratingValue( mode );
					}

					int change;
					double winlose1, winlose2;
					PersonalData pdtmp;
					double double_tmp;
					change = 1;
					while( change > 0 ){
						change = 0;
						for( i = 0; i <= pd.size() - 2; i++ ){
							winlose1 = rank_ptr[i];
							winlose2 = rank_ptr[i + 1];
							if( winlose1 < winlose2 ){
								pdtmp = pd.elementAt( i );
								pd.setElementAt( pd.elementAt( i + 1 ), i );
								pd.setElementAt( pdtmp, i + 1 );
								double_tmp = rank_ptr[i];
								rank_ptr[i] = rank_ptr[i + 1];
								rank_ptr[i + 1] = double_tmp;
								change += 1;
							}
						}
					}

					// 順位の設定
					int rank[] = new int[pd.size()];
					int total = 0;
					double winlose_past;
					rank[0] = 1;
					total = 1;
					winlose_past = pd.elementAt( 0 ).ratingValue( mode );
					double winlose;
					for( i = 1; i <= pd.size() - 1; i++ ){
						winlose = pd.elementAt( i ).ratingValue( mode );
						if( winlose_past == winlose ){
							rank[i] = rank[i - 1];
							total += 1;
						}else{
							rank[i] = total + 1;
							total += 1;
						}
						winlose_past = winlose;
					}
					if( target_lord == Lord.Unknown ){
						PersonalData tmppd;
						for( i = 0; i <= pd.size() - 1; i++ ){
							tmppd = pd.elementAt( i );
							switch( mode ){
								case TOTAL:
									tmppd.orderTotal = rank[i];
									break;
								case DOCTOR:
									tmppd.orderDoctor = rank[i];
									break;
							}
							pd.setElementAt( tmppd, i );
						}
					}

					// 出力
					for( i = 0; i < pd.size(); i++ ){
						makeHtmlKrumaWeeklyOutputPersonal( sw,
								pd.elementAt( i ), rank[i], target_lord, mode );
					}
				}
				sw.write( "</table>" );
				sw.newLine();

				sw.write( "<hr>" );
				sw.newLine();
				sw.write( navigation_bar );
				sw.newLine();
				addAffiliate( sw );
				sw.write( "</body></html>" );
				sw.newLine();
				sw.close();
				// System.out.println( "Html.makeHtmlKrumaWeeklyOutput;file
				// closed" );
				tantra.uploader_main
						.registerUpload( tantra.settings.html_location
								+ File.separator + html_name );
			}catch( Exception e ){
				tantra.printError( e );
			}
		}
	}


	/**
	 * 一人分のPersonalDataを指定されたStreamに書き込みます。 <code>target_lord</code>と<code>mode</code>が一致しなければ、出力しません
	 * 
	 * @param unit
	 *        書き込み対象のfile stream
	 * @param pd
	 *        出力対象のPersonalData
	 * @param rank
	 *        順位
	 * @param target_lord
	 *        出力対象のLord
	 * @param mode
	 *        出力モード
	 */
	private static void makeHtmlKrumaWeeklyOutputPersonal( BufferedWriter unit,
			PersonalData pd, int rank, Lord target_lord, tantra.OutputMode mode ){

		// String comment = "";
		Lord vLord;
		String vLordStr;
		String putline = "";
		try{
			putline = "<tr>" + "<td>" + rank + "位</td><td class=n>" + pd.name + "</td>";
			int index = PersonalData.indexOf( tantra.kruma, pd.name, false );
			vLord = tantra.kruma.elementAt( index ).detectLord();
			vLordStr = vLord.toString();
			putline += "<td";
			switch( vLord ){
				case Unknown:
					putline += ">";
					break;
				case Shiva:
					putline += " class=s>";
					break;
				case Brahma:
					putline += " class=b>";
					break;
				case Vishnu:
					putline += " class=v>";
					break;
			}
			putline += vLordStr + "</td>";
			putline += "<td class=n>"
					+ tantra.joblist.getJob( tantra.kruma.elementAt( index ).name ).toString() + "</td>";
			if( target_lord != Lord.Unknown ){
				if( mode == tantra.OutputMode.DOCTOR ){
					putline += "<td>" + pd.orderDoctor + "位</td>";
				}else{
					putline += "<td>" + pd.orderTotal + "位</td>";
				}
			}
			putline += "<td>" + pd.win + "</td>";
			putline += "<td>" + pd.lose + "</td>";
			double DMG, Doctor;
			DMG = pd.dmg();
			Doctor = pd.doctor();
			double winlose1;
			winlose1 = DMG + Doctor;
			DecimalFormat winlose = new DecimalFormat( "####0.0" );
			// DecimalFormat mins = new DecimalFormat( "時間" );
			DecimalFormat loseperhour = new DecimalFormat( "#####0.000" );
			putline += "<td>" + winlose.format( winlose1 ) + "</td>";
			putline += "<td>" + minsToString( pd.activityMin ) + "</td>";
			putline += "<td>"
					+ loseperhour.format( (double) pd.lose / (pd.activityMin
							/ 60.0) ) + "</td>";
			putline += "<td>" + winlose.format( DMG ) + "</td>";
			putline += "<td>" + winlose.format( Doctor ) + "</td>";
			putline += "</tr>";
			unit.write( putline );
			unit.newLine();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * 分を時間に変換した文字列を返します。
	 * 
	 * @param mins
	 *        変換する分数。ex.120
	 * @return 時間に変換した文字列。ex."2.00時間"
	 */
	private static String minsToString( double mins ){
		DecimalFormat fmt = new DecimalFormat( "###0.00" );
		return fmt.format( mins / 60.0 ) + "時間";
	}

}


/**
 * 拡張子がhtmlのファイルにマッチするファイルフィルタ
 * 
 * @author Luft
 */
class FilterHtml implements FilenameFilter {

	public boolean accept( File path, String name ){
		String extent = Path.getExtension( name );
		if( extent.equals( ".html" ) ){
			return true;
		}
		return false;
	}

}
