package tantra;

import java.io.*;

/**
 * ファイルパスの操作など。拡張子のみの取り出し、フルパスからのファイル名の取り出しなど
 * 
 * @author Luft
 * 
 */
public class Path {

	private final static String yen = File.separator;


	/**
	 * テスト用
	 */
	public static void test(){
		String path = "C:\\My Documents\\My Pictures\\サンプル.jpg";
		System.out.println( "GetDirectoryName=" + getDirectoryName( path ) );
		System.out.println( "GetExtension=" + getExtension( path ) );
		System.out.println( "GetFileName=" + getFileName( path ) );
		System.out.println( "GetFileNameWithoutExtension="
				+ getFileNameWithoutExtension( path ) );
	}


	/**
	 * ディレクトリ名の取得
	 * 
	 * @param istring
	 *        パス。ex."C:\My Documents\My Pictures\サンプル.jpg"
	 * @return ディレクトリ名。ex."C:\My Documents\My Pictures"
	 */
	public static String getDirectoryName( String istring ){
		String token[] = istring.split( "\\" + yen );
		String result;
		int i;
		result = token[0];
		for( i = 1; i <= token.length - 2; i++ ){
			result = result + "\\" + token[i];
		}
		return result;
	}


	/**
	 * 拡張子の取得
	 * 
	 * @param istring
	 *        パス。ex."C:\My Documents\My Pictures\サンプル.jpg"
	 * @return 拡張子。ex.".jpg"
	 */
	public static String getExtension( String istring ){
		String token[] = istring.split( "\\" + yen );
		String tmp = token[token.length - 1];
		String token2[] = tmp.split( "\\." );
		tmp = ".";
		return tmp.concat( token2[token2.length - 1] );
	}


	/**
	 * ファイル名の取得
	 * 
	 * @param istring
	 *        パス。ex."C:\My Documents\My Pictures\サンプル.jpg"
	 * @return ファイル名。ex."サンプル.jpg"
	 */
	public static String getFileName( String istring ){
		String token[] = istring.split( "\\" + yen );
		return token[token.length - 1];
	}


	/**
	 * ファイル名（拡張子なし）の取得
	 * 
	 * @param istring
	 *        パス。"C:\My Documents\My Pictures\サンプル.jpg"
	 * @return 拡張子なしのファイル名。ex."サンプル"
	 */
	public static String getFileNameWithoutExtension( String istring ){
		String token[] = istring.split( "\\" + yen );
		String tmp = token[token.length - 1];
		String token2[] = tmp.split( "\\." );
		return token2[token2.length - 2];
	}


	/**
	 * ルートディレクトリ名の取得
	 * 
	 * @param istring
	 *        パス。ex."C:\My Documents\My Pictures\サンプル.jpg"
	 * @return ルートディレクトリ。ex."c:\"
	 */
	public static String getPathRoot( String istring ){
		return "";
	}

}
