package tantra;

import tantra.DateEx.*;

/**
 * 解析ツールの設定データを処理するためのクラス。
 * 
 * @author Luft
 * 
 */
public class Settings {

	/** 最後に読み込んだtotal dataの行を格納します */
	public String last_read_line;
	/** ChatLogが保存されるパスを格納します */
	public String chat_log_location;
	/* 編集用中間ファイルの保存場所 */
	//public String edit1_location;
	/** バックアップする場所 */
	public String backup_location;
	/** 予備解析データなどの保管場所 */
	public String total_data_location;
	/** htmlファイルを保存する場所 */
	public String html_location;
	/** ChatLogを検査する時間間隔をミリ秒単位で格納します */
	public int listening_interval;
	/** 最後に読み込んだChatLogの行を格納します */
	public String last_read_log;
	/** データを取る対象の聖地の種類 */
	public String target_kruma;
	/** UWSCファイルがある場所 */
	public String uwsc_location;
	/** 最後に読み込まれたUWSCLOGENTRYのレコード位置 */
	public long last_uwsc_log_entry;
	/** htmlファイルなどをアップロードするFTPサーバの設定 */
	public FTPSetting ftp_main;
	/** バックアップをアップロードするFTPサーバの設定 */
	public FTPSetting ftp_backup;
	/** 最後にバックアップが作成された日時 */
	public DateEx last_zipped_date;


	public Settings() {
		last_read_line = "";
		chat_log_location = "C:\\Program Files\\Tantra\\ScreenShot";
		//edit1_location = "C:\\Program Files\\Tantra\\ScreenShot\\edit1";
		backup_location = "C:\\Program Files\\Tantra\\ScreenShot\\backup";
		total_data_location = "C:\\Documents and Settings\\All Users\\Documents\\Tantra_hack";
		html_location = "C:\\Documents and Settings\\All Users\\Documents\\Tantra_hack\\html";
		listening_interval = 60000;
		last_read_log = "";
		target_kruma = "";
		uwsc_location = "C:\\Documents and Settings\\All Users\\Documents\\Tantra_hack\\tantra_data";
		last_uwsc_log_entry = 0;
		ftp_main = new FTPSetting();
		ftp_backup = new FTPSetting();
		last_zipped_date = new DateEx();
	}


	/**
	 * 設定値をtantra_data.iniに書き込みます
	 * 
	 * @param fpath
	 *        保存するファイルへのパス。
	 */
	public void write( String fpath ){
		try{
			IniFile.setEncoding( tantra.ShiftJIS );
			IniFile.WritePrivateProfileString( "", "last_read_line", last_read_line, fpath );
			IniFile.WritePrivateProfileString( "", "chat_log_location", chat_log_location, fpath );
			IniFile.WritePrivateProfileString( "", "backup_location", backup_location, fpath );
			IniFile.WritePrivateProfileString( "", "total_data_location", total_data_location, fpath );
			IniFile.WritePrivateProfileString( "", "html_location", html_location, fpath );
			IniFile.WritePrivateProfileString( "", "last_read_log", last_read_log, fpath );
			IniFile.WritePrivateProfileString( "", "target_kruma", target_kruma, fpath );
			IniFile.WritePrivateProfileString( "", "uwsc_location", uwsc_location, fpath );
			IniFile.WritePrivateProfileString( "", "last_zipped_date", last_zipped_date.toString( Format.yyyyMMdd ), fpath );
			IniFile.WritePrivateProfileString( "", "listening_interval", "" + listening_interval, fpath );
			IniFile.WritePrivateProfileString( "", "last_uwsc_log_entry", "" + last_uwsc_log_entry, fpath );

			IniFile.WritePrivateProfileString( "ftp_main", "host", ftp_main.host, fpath );
			IniFile.WritePrivateProfileString( "ftp_main", "path_on_server", ftp_main.path_on_server, fpath );
			IniFile.WritePrivateProfileString( "ftp_main", "user_id", ftp_main.user_id, fpath );
			IniFile.WritePrivateProfileString( "ftp_main", "user_pw", ftp_main.user_pw, fpath );

			IniFile.WritePrivateProfileString( "ftp_backup", "host", ftp_backup.host, fpath );
			IniFile.WritePrivateProfileString( "ftp_backup", "path_on_server", ftp_backup.path_on_server, fpath );
			IniFile.WritePrivateProfileString( "ftp_backup", "user_id", ftp_backup.user_id, fpath );
			IniFile.WritePrivateProfileString( "ftp_backup", "user_pw", ftp_backup.user_pw, fpath );

		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * 前回終了時の設定値を，tantra_data.iniから読み込みます
	 * 
	 * @param fpath
	 *        設定値が保存されたファイルへのパス
	 */
	public void read( String fpath ){
		if( !tantra.exists( fpath ) ){
			tantra.pushConsoleLine( "error:tantra_data.ini does not exists." );
			System.exit( 0 );
		}
		try{
			Settings _default = new Settings();
			String line;
			last_read_line = IniFile.GetPrivateProfileString( "", "last_read_line", _default.last_read_line, fpath );
			chat_log_location = IniFile.GetPrivateProfileString( "", "chat_log_location", _default.chat_log_location, fpath );
			backup_location = IniFile.GetPrivateProfileString( "", "backup_location", _default.backup_location, fpath );
			total_data_location = IniFile.GetPrivateProfileString( "", "total_data_location", _default.total_data_location, fpath );
			html_location = IniFile.GetPrivateProfileString( "", "html_location", _default.html_location, fpath );
			last_read_log = IniFile.GetPrivateProfileString( "", "last_read_log", _default.last_read_log, fpath );
			target_kruma = IniFile.GetPrivateProfileString( "", "target_kruma", _default.target_kruma, fpath );
			uwsc_location = IniFile.GetPrivateProfileString( "", "uwsc_location", _default.uwsc_location, fpath );
			line = IniFile.GetPrivateProfileString( "", "last_zipped_date", _default.last_zipped_date.toString( Format.yyyyMMdd ), fpath );
			listening_interval = IniFile.GetPrivateProfileInt( "", "listening_interval", _default.listening_interval, fpath );
			line = IniFile.GetPrivateProfileString( "", "last_uwsc_log_entry", "" + _default.last_uwsc_log_entry, fpath );
			last_uwsc_log_entry = Integer.valueOf( line.trim() );
			ftp_main.host = IniFile.GetPrivateProfileString( "ftp_main", "host", "", fpath );
			ftp_main.user_id = IniFile.GetPrivateProfileString( "ftp_main", "user_id", "", fpath );
			ftp_main.user_pw = IniFile.GetPrivateProfileString( "ftp_main", "user_pw", "", fpath );
			ftp_main.path_on_server = IniFile.GetPrivateProfileString( "ftp_main", "path_on_server", "", fpath );
			ftp_backup.host = IniFile.GetPrivateProfileString( "ftp_backup", "host", "", fpath );
			ftp_backup.user_id = IniFile.GetPrivateProfileString( "ftp_backup", "user_id", "", fpath );
			ftp_backup.user_pw = IniFile.GetPrivateProfileString( "ftp_backup", "user_pw", "", fpath );
			ftp_backup.path_on_server = IniFile.GetPrivateProfileString( "ftp_backup", "path_on_server", "", fpath );
		}catch( Exception e ){
			tantra.printError( e );
			System.exit( 0 );
		}
	}

}