package tantra;

import java.io.*;
import java.text.DecimalFormat;

/**
 * バイナリ形式のTOTAL DATAの読み書きを行います。
 * @author Luft
 * @since 2.1
 */
public class TotalDataServer{
	/** total dataへのパス */
	private String fpath;
	private RandomAccessFile body = null;
	private RandomAccessFile indx = null;
	/**  */
	private int position = 0;

	public boolean open( String fpath ){
		this.fpath = fpath;
		if( !tantra.exists( fpath ) ) return false;
		try{
			body = new RandomAccessFile( this.fpath, "rw" );
			indx = new RandomAccessFile( this.fpath + ".indx", "rw" );
			return true;
		}catch( Exception e ){
			tantra.printError( e );
			try{
				if( body != null ) body.close();
				if( indx != null ) indx.close();
			}catch( Exception ex ){}
			return false;
		}
	}


	public void close(){
		try{
			if( body != null ) body.close();
			if( indx != null ) indx.close();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * 
	 * @return
	 */
	public String readLine(){
		int location = 4 + 4 * position;
		try{
			indx.seek( location );
			int type = indx.read();
			int length = indx.read();
			byte[] buffer = new byte[length];
			indx.read( buffer );
			String result = generateLine( type, buffer );
			this.position += 1;
			return result;
		}catch( Exception e ){
			tantra.printError( e );
			return "";
		}
	}


	public void seek( int position ){
		this.position = position;
	}


	public void write(){
		
	}

	
	public TotalDataEntry read(){
		return new TotalDataEntry();
	}


	/**
	 * 
	 * @param type
	 * @param buffer
	 * @return
	 */
	private String generateLine( int type, byte[] buffer ){
		int name1_len, name2_len, lord;
		String name1, name2;
		long date;
		Lord ld;
		try{
			if( type == 1 ){
				name1_len = buffer[0];
				name1 = new String( buffer, 1, name1_len, tantra.ShiftJIS );
				name2_len = buffer[1 + name1_len];
				name2 = new String( buffer, 1 + name1_len, name2_len, tantra.ShiftJIS );
				date = bytes2long( buffer, 1 + name1_len + name2_len );
				return "" + type + "," + name1 + "," + name2 + "," + long2DateString( date );
			}else if( type == 2 ){
				lord = buffer[0];
				ld = Lord.fromInt( lord );
				name1_len = buffer[1];
				name1 = new String( buffer, 2, name1_len, tantra.ShiftJIS );
				date = bytes2long( buffer, 2 + name1_len );
				return "" + type + "," + ld.toString() + "," + name1 + "," + long2DateString( date );
			}else if( type == 3 || type == 4 || type == 5 ){
				name1_len = buffer[0];
				name1 = new String( buffer, 1, name1_len );
				lord = buffer[1 + name1_len];
				ld = Lord.fromInt( lord );
				date = bytes2long( buffer, 2 + name1_len );
				return "" + type + "," + name1 + "," + ld.toString() + "," + long2DateString( date );
			}else{
				return "";
			}
		}catch( Exception e ){
			tantra.printError( e );
			return "";
		}
	}


	private static long bytes2long( byte[] buffer, int offset ){
		byte b1, b2, b3, b4, b5, b6, b7, b8;
		b1 = buffer[offset];
		b2 = buffer[offset + 1];
		b3 = buffer[offset + 2];
		b4 = buffer[offset + 3];
		b5 = buffer[offset + 4];
		b6 = buffer[offset + 5];
		b7 = buffer[offset + 6];
		b8 = buffer[offset + 7];
		return ((long)b1 << 56) + ((long)b2 << 48)
	     + ((long)b3 << 40) + ((long)b4 << 32)
	     + ((long)b5 << 24) + ((long)b6 << 16)
	     + ((long)b7 << 8) + b8;
	}


	private static String long2DateString( long value ){
		long ymd, hour, min, sec, v;
		DecimalFormat fmt8 = new DecimalFormat( "00000000" );
		DecimalFormat fmt2 = new DecimalFormat( "00" );
		v = value;
		ymd = v / 1000000L;
		v = v - ymd * 1000000L;
		hour = v / 10000L;
		v = v - hour * 10000L;
		min = v / 100L;
		sec = v - min * 100L;
		return fmt8.format( ymd ) + "," + fmt2.format( hour ) + ":" + fmt2.format( min ) + ":" + fmt2.format( sec );
	}
}
