package tantra;

/**
 * UWSCが出力したログファイルを解析
 * 
 * @author Luft
 * 
 */
public class UwscLogEntry {

	private String entry_type;
	private String year;
	private String month;
	private String day;
	private String hour;
	private String minute;
	private String second;

	public enum EntryType {
		START, CONTINUE, END, UNKNOWN
	}


	public EntryType entryType(){
		if( entry_type.startsWith( "0" ) ){
			return EntryType.START;
		}else if( entry_type.startsWith( "1" ) ){
			return EntryType.CONTINUE;
		}else if( entry_type.startsWith( "2" ) ){
			return EntryType.END;
		}else{
			return EntryType.UNKNOWN;
		}
	}


	public UwscLogEntry() {
		entry_type = "Z";
		year = "0000";
		month = "00";
		day = "00";
		hour = "00";
		minute = "00";
		second = "00";
	}


	/**
	 * uwsc logの行データからのコンストラクタ
	 * 
	 * @param line
	 */
	public UwscLogEntry( String line ) {
		// ex. line = "0,20070401 02:19:34"
		// UwscLogEntry entry = new UwscLogEntry();
		entry_type = line.substring( 0, 1 );
		year = line.substring( 2, 6 );
		month = line.substring( 6, 8 );
		day = line.substring( 8, 10 );
		hour = line.substring( 11, 13 );
		minute = line.substring( 14, 16 );
		second = line.substring( 17, 19 );
		// return entry;
	}


	/**
	 * このエントリーが<var>d</var>と等しいか否かを判定します
	 * 
	 * @param d
	 *        比較対象のUwscLogEntry
	 * @return このエントリーが<code>d</code>と等しければtrueを、そうでなければfalseを返します。
	 */
	public boolean equals( UwscLogEntry d ){
		if( entry_type.equals( d.entry_type ) && year.equals( d.year )
				&& month.equals( d.month ) && day.equals( d.day )
				&& hour.equals( d.hour ) && minute.equals( d.minute )
				&& second.equals( d.second ) ){
			return true;
		}
		return false;
	}


	/**
	 * このUwscLogEntryがnullであるかどうかを判定します
	 * 
	 * @return nullであればtrueを、そうでなければfalseを返します。
	 */
	public boolean isNull(){
		if( equals( new UwscLogEntry() ) ){
			return true;
		}else{
			return false;
		}
	}


	/**
	 * UWSCログエントリーに記録されている日時情報を取り出します
	 * 
	 * @return
	 */
	public DateEx date(){
		if( equals( new UwscLogEntry() ) ){
			return new DateEx();
		}else{
			int y, m, d, h, mi, s;
			y = Integer.valueOf( year );
			m = Integer.valueOf( month );
			d = Integer.valueOf( day );
			h = Integer.valueOf( hour );
			mi = Integer.valueOf( minute );
			s = Integer.valueOf( second );
			if( DateEx.isValidDate( y, m, d, h, mi, s )){
				return new DateEx( y, m, d, h, mi, s );
			}else{
				return new DateEx();
			}
		}
	}

}