package tantra;

import java.io.*;
import java.util.*;
import tantra.DateEx.*;

/**
 * 一週間分のUWSCの稼動状況に関する情報を取り扱う
 * 
 * @author Luft
 */
public class WeeklyRunningInfo {

	public Vector<RunningInfoEntry> weekly_running_info = new Vector<RunningInfoEntry>();
	private DateEx currentSunday = new DateEx();


	/**
	 * 現在保持している一週間分の解析実行状況の情報を、別の週の情報を保持させるためにリセットします。
	 * 
	 * @param week_start
	 *        新たに保持させる週の日曜日の日付。
	 */
	public void initialize( DateEx week_start ){
		currentSunday = week_start.clone();

		if( weekly_running_info.size() > 0 ){
			weekly_running_info.removeAllElements();
		}
		write();

	}


	/**
	 * <var>ioDailyRunningInfo</var>に，<var>start</var>から解析が始動したという情報を追加します
	 * 
	 * @param start
	 *        日付
	 */
	private void start( DateEx start ){
		// 追加する位置を探す
		/*
		 * DateEx search = new DateEx( start.year(), start.month(), start.day(),
		 * 0, 0, 0 ); int i, index;
		 */
		/*
		 * DailyRunningInfo ioDailyRunningInfo = new DailyRunningInfo(
		 * DateEx.DATE_NULL );
		 */
		/*
		 * index = -1; for( i = 0; i <= 6; i++ ){ if(
		 * weekly_running_info.elementAt( i ).pile.getSumYMD() ==
		 * search.getSumYMD() ){ index = i; break; } } System.out.println(
		 * "WeeklyRunningInfo::addRunningInfoEntryStarts;index=" + index ); if(
		 * index < 0 ) return;
		 */
		if( currentSunday.ymd() < start.weekStart().ymd() ){
			createHtml();
			initialize( start.weekStart() );
		}

		if( weekly_running_info.size() <= 0 ){
			weekly_running_info
					.add( new RunningInfoEntry( start, new DateEx() ) );
			// System.out.println( "first element added" );
		}else{
			if( weekly_running_info.lastElement().to.largerThan( start ) ){
				// 追加しようとしている日付が，既に保持されている日付よりも新しい
				// 何もしない
				// System.out.println( "to larger than start" );
				return;
			}
			weekly_running_info
					.add( new RunningInfoEntry( start, new DateEx() ) );
			// System.out.println( "normal operation" );
		}

		// 最後に、weekly_running_infoを更新してreturn
		// weekly_running_info.setElementAt( ioDailyRunningInfo, index );
	}


	/**
	 * 指定された日時に、UWSCが停止したという情報を、現在のUWSC稼動状況データに追加します。
	 * 
	 * @param end
	 *        日付
	 */
	private void end( DateEx end ){
		// 追加する対象を探す
		/*
		 * DateEx search = new DateEx( end.year(), end.month(), end.day(), 0, 0,
		 * 0 ); int i, index;
		 */
		/*
		 * DailyRunningInfo ioDailyRunningInfo = new DailyRunningInfo(
		 * DateEx.DATE_NULL );
		 */
		/*
		 * index = -1; for( i = 0; i <= 6; i++ ){ if(
		 * weekly_running_info.elementAt( i ).pile.getSumYMD() ==
		 * search.getSumYMD() ){ // ioDailyRunningInfo =
		 * weekly_running_info.elementAt( i ); index = i; break; } }
		 * System.out.println(
		 * "WeeklyRunningInfo::addRunningInfoEntryEnds;index=" + index ); if(
		 * index < 0 ) return;
		 */

		if( weekly_running_info.size() <= 0 ){
			weekly_running_info.add( new RunningInfoEntry( new DateEx(), end ) );
			// System.out.println( "first element added" );
		}else{
			// int length = ioDailyRunningInfo.entry.size() - 1;
			if( weekly_running_info.lastElement().from.largerThan( end ) ){
				// 追加しようとした終了日付情報が，既に保持されている開始日付情報よりも古い場合
				// System.out.println( "from is larger than end" );
				return;
			}
			weekly_running_info.lastElement().to = end;
			// System.out.println( "normal operation" );
		}
		// weekly_running_info.setElementAt( ioDailyRunningInfo, index );
	}


	/**
	 * この関数が呼ばれた時点でのweekly_running_info配列の内容をファイルに保存します
	 */
	public void write(){
		try{
			BufferedWriter sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( tantra.settings.total_data_location
							+ File.separator + tantra.WEEKLYRUNNINGINFO
							+ ".txt" ), tantra.ShiftJIS ) );
			int j, length;
			length = weekly_running_info.size();
			sw.write( currentSunday.toString( Format.yyyyMMdd ) + ","
					+ length );
			sw.newLine();
			if( length >= 0 ){
				for( j = 0; j < length; j++ ){
					sw.write( weekly_running_info.elementAt( j ).from
							.toString( Format.yyyyMMdd )
							+ weekly_running_info.elementAt( j ).from
									.toString( Format.hhmmss )
							+ ","
							+ weekly_running_info.elementAt( j ).to
									.toString( Format.yyyyMMdd )
							+ weekly_running_info.elementAt( j ).to
									.toString( Format.hhmmss ) );
					sw.newLine();
				}
			}
			sw.close();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * 前回実行時のweekly_running_info配列の内容をファイルから読み込み，引き継ぎます
	 */
	public void read(){
		String fname = tantra.settings.total_data_location + File.separator
				+ tantra.WEEKLYRUNNINGINFO + ".txt";
		if( !tantra.exists( fname ) ) return;

		try{
			BufferedReader sr = new BufferedReader( new InputStreamReader(
					new FileInputStream( fname ), tantra.ShiftJIS ) );
			int j, length;
			String line;
			String token[];
			// for( i = 0; i <= 6; i++ ){
			// DailyRunningInfo dri = weekly_running_info.elementAt( i );
			// With weekly_running_info(i)
			if( (line = sr.readLine()) == null ){
				sr.close();
				return;
			}
			token = line.split( "," );
			currentSunday = new DateEx( token[0] + "000000" );
			length = Integer.valueOf( token[1] ) - 1;
			DateEx from, to;
			weekly_running_info.removeAllElements();
			if( length >= 0 ){
				for( j = 0; j <= length; j++ ){
					line = sr.readLine();
					token = line.split( "," );
					from = new DateEx( token[0] );
					to = new DateEx( token[1] );
					weekly_running_info.add( new RunningInfoEntry( from, to ) );
				}
			}
			// End With
			// }
			sr.close();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * UWSCのログエントリーを処理し、現在のUWSC稼動状況データを更新します。
	 * 
	 * @param entry
	 *        処理するエントリー
	 */
	public void analyze( UwscLogEntry entry ){
		DateEx d = entry.date();
		DateEx week_start = d.weekStart();
		if( currentSunday.ymd() < week_start.ymd() ){
			createHtml();
			initialize( week_start );
			currentSunday = week_start;
		}
		switch( entry.entryType() ){
			case START:
				start( d );
				break;
			case CONTINUE:
				end( d );
				break;
			case END:
				end( d );
				break;
		}
	}


	/**
	 * 現在保持しているUWSCの稼動状況データを、HTMLファイルに出力します。
	 */
	public void createHtml(){
		String html_name;
		DateEx d = currentSunday;

		if( d.year() < 2000 ) return;

		html_name = tantra.settings.html_location + File.separator
				+ "running_info_" + tantra.settings.target_kruma + "_"
				+ d.toString( Format.yyyyMMdd ) + ".html";
		if( !tantra.fixed.isFixed( html_name ) ){
			// System.out.println( "html_name is not fixed" );
			try{
				BufferedWriter unit = new BufferedWriter(
						new OutputStreamWriter(
								new FileOutputStream( html_name ),
								tantra.ShiftJIS ) );
				int length, j;
				Html.writeHeader( unit, "running_info "
						+ d.toString( Format.yyyy_MM_dd ) );
				unit.write( "<body>" );
				unit.newLine();
				String navigation;
				navigation = "<p>"
						+ Html.LINK_TOP
						+ Html.GT
						+ Html.tagA( "index_" + tantra.settings.target_kruma
								+ ".html", tantra.settings.target_kruma )
						+ Html.GT + d.toString( Format.yyyy_MM_dd ) + Html.GT
						+ "running_info</p>";
				unit.write( navigation );
				unit.newLine();
				// String line;
				// String token[];
				unit
						.write( "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">" );
				unit.newLine();
				unit.write( "<tr>" );
				unit.newLine();
				unit
						.write( "  <td class=\"title\" colspan=\"2\">欠測時間等の情報</td>" );
				unit.newLine();
				unit.write( "</tr>" );
				// for( i = 0; i <= 6; i++ ){
				if( weekly_running_info.size() <= 0 ){
					length = -1;
				}else{
					length = weekly_running_info.size() - 1;
				}
				if( length >= 0 ){
					for( j = 0; j <= length; j++ ){
						RunningInfoEntry entry = weekly_running_info
								.elementAt( j );
						// With weekly_running_info(i).entry(j)
						unit.write( "<tr>" );
						unit.newLine();
						unit.write( "  <td>ログ採取</td>" );
						unit.newLine();
						unit.write( "  <td>" );
						if( !entry.from.isNull() ){
							unit.write( entry.from.toString( Format.yyyy_MM_dd )
									+ ":"
									+ entry.from.toString( Format.hh_mm_ss )
									+ "&nbsp;～&nbsp;" );
						}else{
							unit.write( "&nbsp;～&nbsp;" );
						}
						if( !entry.to.isNull() ){
							unit
									.write( entry.to
											.toString( Format.yyyy_MM_dd )
											+ ":"
											+ entry.to
													.toString( Format.hh_mm_ss ) );
						}
						unit.write( "</td>" );
						unit.newLine();
						unit.write( "</tr>" );
						unit.newLine();
						// End With
					}
				}
				// }

				unit.write( "</table>" );
				unit.newLine();
				unit.write( "<hr>" );
				unit.newLine();
				unit.write( navigation );
				unit.newLine();
				Html.addAffiliate( unit );
				unit.write( "</body>" );
				unit.newLine();
				unit.write( "</html>" );
				unit.newLine();
				unit.close();
				// System.out.println(
				// "WeeklyRunningInfo::uwscRebuildRunningInfo;output finished"
				// );
				tantra.uploader_main.registerUpload( html_name );
				// System.out.println(
				// "WeeklyRunningInfo::uwscRebuildRunningInfo;upload registered"
				// );
			}catch( Exception e ){
				tantra.printError( e );
			}
		}
	}
}


class RunningInfoEntry {

	public DateEx from;
	public DateEx to;


	public RunningInfoEntry( DateEx from, DateEx to ) {
		this.from = from;
		this.to = to;
	}

}
