package tantra;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * FTPファイル送信サンプル PASVモードでの接続を試み、PASVモードが使用できない場合は、 ACTIVEモードでファイルを送信
 * 
 * @author <a href="http://melrose.jugem.cc/">stroll</a>
 * @author Luft
 * @version 1.0
 */
public class ftp {

	/** ftpの制御用のポート */
	private static final int CTRLPORT = 21;
	/** 制御用ソケット */
	private static Socket ctrlSocket;
	/** 制御出力用ストリーム */
	private static PrintWriter ctrlOutput;
	/** 制御入力用ストリーム */
	private static BufferedReader ctrlInput;
	/** ローカルホストのアドレス */
	private static byte[] localHostAddress;
	/** ホスト名 */
	private String hostName;
	/** ユーザ名 */
	private String userName;
	/** パスワード */
	private String password;


	public ftp( String hostName, String userName, String password,
			String dirName ) {
		this.hostName = hostName;
		this.userName = userName;
		this.password = password;
	}


	/**
	 * Vector<String>に格納されたファイルを、FTPアップロードします
	 * @param files
	 * 		アップロードするファイルのリスト
	 * @return
	 * 		転送に成功すれば<code>true</code>を、失敗すれば<code>false</code>を返します
	 */
	public boolean upload( Vector<FileEntry> files ){
		try{
			// String fileName = "temp"; // 転送ファイル
			// String storFileName = "ftptest.txt"; // 転送後のファイル名

			// 接続
			ctrlSocket = new Socket( hostName, CTRLPORT );
			localHostAddress = ctrlSocket.getLocalAddress().getAddress();
			ctrlOutput = new PrintWriter(
					new BufferedWriter( new OutputStreamWriter( ctrlSocket
							.getOutputStream(), "JIS" ) ), true );
			ctrlInput = new BufferedReader( new InputStreamReader( ctrlSocket
					.getInputStream() ) );

			String line = readTillEnd();

			// ユーザー認証
			ctrlOutput.println( "USER " + userName );
			ctrlOutput.flush();
			line = readTillEnd();

			ctrlOutput.println( "PASS " + password );
			ctrlOutput.flush();
			line = readTillEnd();

			// 指定したディレクトリに移動
			// ctrlOutput.println( "CWD " + dirName.replace( "/", "" ) );
			// ctrlOutput.flush();
			// line = readTillEnd();

			// バイナリモードを設定(アスキーモードの場合は'TYPE A')
			ctrlOutput.println( "TYPE I" );
			ctrlOutput.flush();
			line = readTillEnd();

			// PASVモードテスト
			int j;
			for( j = 0; j < files.size(); j++ ){
				String fileName = files.elementAt( j ).name;
				String storFileName = Path.getFileName( fileName );
				File f = new File( fileName );
				if( f.exists() ){
					ctrlOutput.println( "PASV" );
					ctrlOutput.flush();
					line = readTillEnd();

					// PASVモード可の場合
					if( Integer.parseInt( line.substring( 0, 3 ) ) == 227 ){
						// System.out.println( "PASVモード" );
						int port;
						String ip;
						int[] numbers = new int[6];
						String temp;
						StringTokenizer tempToken;
						tempToken = new StringTokenizer( line, "(" );
						temp = tempToken.nextToken();
						temp = tempToken.nextToken();
						tempToken = new StringTokenizer( temp, ")" );
						temp = tempToken.nextToken();
						tempToken = new StringTokenizer( temp, "," );
						for( int i = 0; i < 6; i++ ){
							numbers[i] = Integer.parseInt( tempToken.nextToken() );
						}
	
						port = (numbers[4] * 256) + numbers[5];
						ip = numbers[0] + "." + numbers[1] + "." + numbers[2] + "."
								+ numbers[3];
	
						InetAddress hostAddress = InetAddress.getByName( ip );
						//byte[] address = hostAddress.getAddress();
						Socket dataSocket = new Socket( hostAddress, port );
						dataSocket.setSoTimeout( 5000 );
	
						ctrlOutput.println( "STOR " + storFileName );
						ctrlOutput.flush();
						line = readTillEnd();
						// System.out.println( "STOR line=" + line );
	
						int n;
						int transferred = 0;
						byte[] buff = new byte[1024];
						DataOutputStream dataOutput = new DataOutputStream(
								dataSocket.getOutputStream() );
						FileInputStream file = new FileInputStream( fileName );
						while( (n = file.read( buff )) > 0 ){
							dataOutput.write( buff, 0, n );
							transferred += n;
						}
						file.close();
						dataOutput.close();
						//line = readTillEnd();
						dataSocket.close();
						line = readTillEnd();
						files.elementAt( j ).movable = true;
					}else{
/*						//System.out.println( "ACTIVEモード" );
						//String fileName = files.elementAt( i );
						//String storFileName = Path.getFileName( fileName );
						FileInputStream fis = new FileInputStream( fileName );
						Socket dataSocket = dataConnection( "STOR " + storFileName );
						OutputStream outstr = dataSocket.getOutputStream();
						int n;
						byte[] buff = new byte[1024];
						while( (n = fis.read( buff )) > 0 ){
							outstr.write( buff, 0, n );
						}
						dataSocket.close();
						fis.close();
						files.elementAt( j ).movable = true;*/
					}
				}
			}

			//line = readTillEnd();
			//System.out.println( "STOR line=" + line );

			ctrlOutput.println( "QUIT" );
			ctrlOutput.flush();
			line = readTillEnd();
			//System.out.println( "QUIT line=" + line );

			// 接続を閉じます
			ctrlOutput.close();
			ctrlInput.close();
			ctrlSocket.close();
			return true;
		}catch( Exception e ){
			tantra.printError( e );
			return false;
		}
	}


	/**
	 * 制御入力用ストリームから、FTPサーバーの応答コードを読み込みます
	 * @return
	 * 		制御入力用ストリームから読み取ったメッセージの最後の行
	 */
	private String readTillEnd(){
		try{
			String line = "";
			while( (line = ctrlInput.readLine()) != null ){
				//System.out.println( line );
				if( line.charAt( 3 ) != '-' ) break;
			}
			return line;
		}catch( Exception e ){
			tantra.printError( e );
			return "";
		}
	}


	/**
	 * データ送受信用ソケットを取得
	 * 
	 * @param ctrlcmd
	 */
/*	private Socket dataConnection( String ctrlcmd ) throws IOException,
			UnknownHostException{
		String cmd = "PORT ";
		ServerSocket serverDataSocket = new ServerSocket( 0, 1 );
		for( int i = 0; i < 4; i++ ){
			cmd = cmd + (localHostAddress[i] & 0xff) + ",";
		}
		cmd += (((serverDataSocket.getLocalPort()) / 256) & 0xff) + ","
				+ (serverDataSocket.getLocalPort() & 0xff);

		ctrlOutput.println( cmd );
		ctrlOutput.flush();
		readTillEnd();
		// System.out.println( "PORT line=" + line );

		ctrlOutput.println( ctrlcmd );
		ctrlOutput.flush();
		readTillEnd();
		Socket dataSocket = serverDataSocket.accept();
		serverDataSocket.close();
		return dataSocket;
	}*/
}