import java.util.*;
import java.io.*;
import tantra.*;
import tantra.DateEx.*;

public class tantra_data {

	/** カレントディレクトリのパス */
	private static String CURRENT_DIR;
	/** tantra_data.txtのためのBufferedWriter */
	private static BufferedWriter TOTAL;
	private static ChatLogServer log_server;
	private static UwscLogServer uwsc_server;
	private static DateEx kruma_weekly_currentSunday;
	private static DateEx current_date;
	/** 読み込んだ行データが過去に解析済みのものであるかどうかをあらわすフラグ */
	private static boolean is_old = false;

	
	/**
	 * エントリーポイント。初期化し、タイマーでループ
	 * 
	 * @param args
	 *        コマンドライン引数。<code>args[0]</code>="debug"なら、テスト用のデータを作成
	 */
	public static void main( String args[] ){

		if( args.length >= 1 ){
			if( args[0].equals( "debug" ) ) tantra.debug = true;
			tantra.createTestData();
		}
		init();
		int count = tantra.settings.listening_interval / 5000;
		int interval = tantra.settings.listening_interval / count;
		int i;
		while( true ){
			try{
				for( i = 0; i < count; i++ ){
					Thread.sleep( interval );
					emergencyExit();
					if( tantra.exists( "wait" ) ){
						// ロックファイルがある場合。データの修正作業が終わるのを待つ。
						writeDump();
						waitEditing();
						readDump();
					}
				}
			}catch( Exception e ){
				tantra.printError( e );
			}
			timerTick();
		}
	}


	/**
	 * 初期化処理。
	 */
	private static void init(){
		CURRENT_DIR = new File( "." ).getAbsoluteFile().getParent();
		if( CURRENT_DIR.endsWith( File.separator ) ){
			CURRENT_DIR = CURRENT_DIR.substring( 0, CURRENT_DIR.length() - 1 );
		}

		try{
			tantra.pushConsoleLine( "tantra_data [version " + tantra.ver + "]" );
			tantra.pushConsoleLine( "init" );
			tantra.settings = new Settings();
			tantra.settings.read( tantra.SETTING_FILE_NAME );
			//readFTPSettings();
			kruma_weekly_currentSunday = DateEx.dateFromTotalData(
					tantra.settings.last_read_line ).weekStart();
			if( new File( tantra.settings.backup_location ).exists() ){
				new File( tantra.settings.backup_location ).mkdir();
			}
			if( new File( tantra.settings.backup_location ).exists() ){
				new File( tantra.settings.backup_location ).mkdir();
			}
			/*
			 * if( new File( tantra.settings.edit1_location ).exists() ){ new
			 * File( tantra.settings.edit1_location ).mkdir(); }
			 */
			if( new File( tantra.settings.html_location ).exists() ){
				new File( tantra.settings.html_location ).mkdir();
			}
			if( new File( tantra.settings.total_data_location ).exists() ){
				new File( tantra.settings.total_data_location ).mkdir();
			}

			// uwsc_current_record = 0;
			if( new File( tantra.settings.uwsc_location + File.separator
					+ tantra.LOCKFILE_EXE ).exists() ){
				new File( tantra.settings.uwsc_location + File.separator
						+ tantra.LOCKFILE_EXE ).delete();
			}

			tantra.suspects = new SuspectList(
					tantra.settings.total_data_location
							+ tantra.SUSPECT_FILE_NAME );
			tantra.suspects.read();
			tantra.fixed = new FixedFileList();
			tantra.fixed.read();

		}catch( Exception ex ){
			tantra.printError( ex );
		}
		if( tantra.settings.target_kruma.equals( "" ) ){
			tantra
					.pushConsoleLine( "target_kruma not set. please confirm configuration and restart." );
			System.exit( 0 );
		}

		if( tantra.settings.ftp_backup.host.equals( "" )
				|| tantra.settings.ftp_main.host.equals( "" ) ){
			tantra
					.pushConsoleLine( "ftp server info. not set. please confirm configuration and restart." );
			System.exit( 0 );
		}

		// is_init_successed = true;

		tantra.uploader_main = new Uploader( tantra.settings.ftp_main );
		tantra.uploader_backup = new Uploader( tantra.settings.ftp_backup );

		uwsc_server = new UwscLogServer();

		DateEx d;
		d = DateEx.dateFromTotalData( tantra.settings.last_read_line )
				.weekStart();
		tantra.running_info = new WeeklyRunningInfo();
		tantra.running_info.initialize( d );
		tantra.pushConsoleLine( "UwscPreAnalyzation..." );
		uwscPreAnalyzation();

		tantra.pushConsoleLine( "Load job list..." );
		tantra.joblist = new JobList();
		tantra.joblist.read();
		// readJobList( tantra.settings.total_data_location + File.separator
		// + tantra.JOB_LIST );

		tantra.pushConsoleLine( "Load Pre-Analyzed Data..." );
		tantra.population = new WeeklyPopulation();
		tantra.population.initialize( d );

		readDump();

		tantra.pushConsoleLine( "PreAnalyzation..." );
		preAnalyzation();
		if( tantra.settings.last_read_line.length() <= 0 ){
			is_old = false;
		}else{
			is_old = true;
		}

		Html.createIndex();

		try{
			TOTAL = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( tantra.settings.total_data_location
							+ File.separator + tantra.TOTAL_DATA_FILE_NAME,
							true ), tantra.ShiftJIS ) );
		}catch( Exception e ){
			tantra.printError( e );
		}

		log_server = new ChatLogServer();
		// System.out.println( "onLoad; return" );

	}


	/**
	 * chat logの行データから、total data形式の行データに変換します。 聖地データに関連しないデータであれば "" を返します。
	 */
	private static String editLine( String line ){
		String result, edit;
		// String token[];

		if( line.startsWith( "[告知]" ) && line.indexOf( "に勝利しました。" ) > 0 ){
			result = "";
			edit = line;
			Splitter spl = new Splitter( edit, "[告知]" );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, " が " );
			result = "1," + spl.tokenAt( 0 );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, " に勝利しました。 (" );
			result = result + "," + spl.tokenAt( 0 );
			edit = spl.tokenAt( 1 );
			edit = edit.replace( ")", "" );
			result = result + "," + edit;
			return result;
		}else if( line.startsWith( "\"[告知]主神" ) && line.indexOf( "の使徒" ) > 0 ){
			result = "";
			edit = line;
			Splitter spl = new Splitter( edit, "\"[告知]主神" );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, "の使徒" );
			result = "2," + spl.tokenAt( 0 );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, "が、" );
			result = result + "," + spl.tokenAt( 0 );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, " " );
			edit = spl.tokenAt( 1 );
			edit = edit.replace( "(", "" );
			edit = edit.replace( ")", "" );
			result = result + "," + edit;
			return result;
		}else if( line.startsWith( "[告知]" ) && line.indexOf( "の祝福を受けました。" ) > 0 ){
			result = "";
			edit = line;
			Splitter spl = new Splitter( edit, "[告知]" );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, "は、主神" );
			result = "3," + spl.tokenAt( 0 );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, "の祝福を受けました。 " );
			result = result + "," + spl.tokenAt( 0 );
			edit = spl.tokenAt( 1 );
			edit = edit.replace( "(", "" );
			edit = edit.replace( ")", "" );
			result = result + "," + edit;
			return result;
		}else if( line.startsWith( "[告知] " ) && line.indexOf( "を選択しました。" ) > 0 ){
			result = "";
			edit = line;
			Splitter spl = new Splitter( edit, "[告知] " );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, "が、主神" );
			result = "4," + spl.tokenAt( 0 );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, "を選択しました。 " );
			result = result + "," + spl.tokenAt( 0 );
			edit = spl.tokenAt( 1 );
			edit = edit.replace( "(", "" );
			edit = edit.replace( ")", "" );
			result = result + "," + edit;
			return result;
		}else if( line.indexOf( "神殿に、貢献値 " ) > 0
				&& line.indexOf( " を献納しました。主神" ) > 0 ){
			edit = line;
			Splitter spl = new Splitter( edit, "神殿に、貢献値 " );
			edit = spl.tokenAt( 0 );
			line = spl.tokenAt( 1 );
			spl = new Splitter( edit, "が " );
			result = "6," + spl.tokenAt( 0 ) + "," + spl.tokenAt( 1 );
			spl = new Splitter( line, " を献納しました。主神" );
			edit = spl.tokenAt( 1 );
			spl = new Splitter( edit, "(" );
			edit = spl.tokenAt( 1 );
			edit = edit.replace( ")", "" );
			result = result + "," + edit;
			return result;
		}else{
			return "";
		}
	}


	/**
	 * ftp設定が保存されたファイルから、ファイルを読み込みます。
	 */
	/*private static void readFTPSettings(){
		String fpath = CURRENT_DIR + File.separator + tantra.PASS_TXT;
		File info = new File( fpath );
		if( info.exists() ){
			try{
				BufferedReader sr = new BufferedReader( new InputStreamReader(
						new FileInputStream( fpath ), tantra.ShiftJIS ) );
				tantra.settings.ftp_main.host = sr.readLine();
				tantra.settings.ftp_main.path_on_server = sr.readLine();
				tantra.settings.ftp_main.user_id = sr.readLine();
				tantra.settings.ftp_main.user_pw = sr.readLine();
				tantra.settings.ftp_backup.host = sr.readLine();
				tantra.settings.ftp_backup.path_on_server = sr.readLine();
				tantra.settings.ftp_backup.user_id = sr.readLine();
				tantra.settings.ftp_backup.user_pw = sr.readLine();
				sr.close();
			}catch( Exception e ){
				tantra.printError( e );
				System.exit( 0 );
			}
		}else{
			tantra.settings.ftp_main = new FTPSetting();
			tantra.settings.ftp_backup = new FTPSetting();
		}
	}*/


	/**
	 * 前回終了時のデータをファイルから読み込み、引き継ぎます。
	 */
	private static void readDump(){
		loadPreAnalyzedData();
		tantra.population.read();
		tantra.running_info.read();
	}


	/**
	 * 解析中のデータをファイルに保存します。
	 */
	private static void writeDump(){
		tantra.population.write();
		tantra.running_info.write();
		writePreAnalyzedData();
	}


	/**
	 * プレ・アナライズされたPersonalDataを読み込みます．
	 * 
	 * @return 読み込みに成功すればtrueを、そうでなければfalseを返します。
	 */
	private static boolean loadPreAnalyzedData(){
		String first_line_1 = "";
		String first_line_2 = "";
		String all, week;
		all = tantra.settings.total_data_location + File.separator
				+ tantra.PRE_ANALYZED_DATA;
		week = tantra.settings.total_data_location + File.separator
				+ tantra.PRE_ANALYZED_DATA_WEEKLY;
		if( loadPreAnalyzedDataCheckCorrect( all, first_line_1 )
				&& loadPreAnalyzedDataCheckCorrect( week, first_line_2 ) ){
			if( !first_line_1.equals( first_line_2 ) ) return false;
		}else{
			tantra.printError( "loadPreAnalyzedData::pre-analyzed data time stamp mismatch:" + first_line_1 + ":" + first_line_2 );
			return false;
		}

		// 読み込む
		loadPreAnalyzedDataLoad( tantra.kruma, all );
		loadPreAnalyzedDataLoad( tantra.kruma_weekly, week );
		tantra.settings.last_read_line = first_line_1;
		kruma_weekly_currentSunday = DateEx.dateFromTotalData(
				tantra.settings.last_read_line ).weekStart();
		return true;
	}


	/**
	 * 予備解析データが書き込まれたファイルが正規のものかをチェックします。 最後まで出力されていれば最後の行が"END"であることを利用
	 * 
	 * @param fpath
	 *        チェックするファイルへのパス。
	 * @param first_line
	 *        データファイルの最初の行
	 * @return 正規のものであればtrueを、そうでなければfalseを返します
	 */
	private static boolean loadPreAnalyzedDataCheckCorrect( String fpath,
			String first_line ){
		if( tantra.exists( fpath ) ){
			try{
				BufferedReader unit = new BufferedReader(
						new InputStreamReader( new FileInputStream( fpath ),
								tantra.ShiftJIS ) );
				String line;
				String final_line = "";
				first_line = unit.readLine();
				while( (line = unit.readLine()) != null ){
					final_line = line;
				}
				unit.close();
				if( !final_line.equals( "END" ) ){
					return false;
				}else{
					return true;
				}
			}catch( Exception e ){
				tantra.printError( e );
			}
		}
		return false;
	}


	/**
	 * 予備解析されたデータをファイルから読み込みます
	 * 
	 * @param target_array
	 *        保存対象のVector<PersonalData>
	 * @param fpath
	 *        保存するファイルへのパス
	 */
	private static void loadPreAnalyzedDataLoad(
			Vector<PersonalData> target_array, String fpath ){
		String line;
		target_array.removeAllElements();
		try{
			BufferedReader unit = new BufferedReader( new InputStreamReader(
					new FileInputStream( fpath ), tantra.ShiftJIS ) );
			line = unit.readLine();
			line = unit.readLine();
			while( (line = unit.readLine()) != null ){
				target_array.add( new PersonalData( line ) );
			}
			unit.close();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * この関数が呼ばれた時点におけるkruma, kruma_weeklyの内容をファイルに保存します
	 * 
	 */
	private static void writePreAnalyzedData(){
		if( tantra.exists( tantra.settings.total_data_location + File.separator
				+ tantra.PRE_ANALYZED_DATA_OLD ) ){
			tantra.kill( tantra.settings.total_data_location + File.separator
					+ tantra.PRE_ANALYZED_DATA_OLD );
		}
		if( tantra.exists( tantra.settings.total_data_location + File.separator
				+ tantra.PRE_ANALYZED_DATA ) ){
			tantra.rename( tantra.settings.total_data_location + File.separator
					+ tantra.PRE_ANALYZED_DATA,
					tantra.settings.total_data_location + File.separator
							+ tantra.PRE_ANALYZED_DATA_OLD );
		}
		if( tantra.exists( tantra.settings.total_data_location + File.separator
				+ tantra.PRE_ANALYZED_DATA_OLD_WEEKLY ) ){
			tantra.kill( tantra.settings.total_data_location + File.separator
					+ tantra.PRE_ANALYZED_DATA_OLD_WEEKLY );
		}
		if( tantra.exists( tantra.settings.total_data_location + File.separator
				+ tantra.PRE_ANALYZED_DATA_WEEKLY ) ){
			tantra.rename( tantra.settings.total_data_location + File.separator
					+ tantra.PRE_ANALYZED_DATA_WEEKLY,
					tantra.settings.total_data_location + File.separator
							+ tantra.PRE_ANALYZED_DATA_OLD_WEEKLY );
		}
		PersonalData.write( tantra.kruma, tantra.settings.total_data_location
				+ File.separator + tantra.PRE_ANALYZED_DATA,
				tantra.settings.last_read_line );
		PersonalData.write( tantra.kruma_weekly,
				tantra.settings.total_data_location + File.separator
						+ tantra.PRE_ANALYZED_DATA_WEEKLY,
				tantra.settings.last_read_line );
	}


	/**
	 * プログラム起動時に実行。すでにあるtotal dataを読み込み，全聖地人リストを利用可能な状態にします
	 */
	private static void preAnalyzation(){
		String line = "";
		String final_line = "";
		DateEx last_read_line_date = new DateEx();
		if( tantra.settings.last_read_line.length() <= 0 ){
			is_old = false;
		}else{
			is_old = true;
			last_read_line_date = DateEx.dateFromTotalData( tantra.settings.last_read_line );
		}
		try{
			String fname = tantra.settings.total_data_location + File.separator
					+ tantra.TOTAL_DATA_FILE_NAME;
			if( !tantra.exists( fname ) ) return;
			BufferedReader sr = new BufferedReader( new InputStreamReader(
					new FileInputStream( fname ), tantra.ShiftJIS ) );
			long lines = 0L;
			while( (line = sr.readLine()) != null ){
				lines += 1L;
				if( lines % 10L == 0L ){
					emergencyExit();
				}
				// System.out.println( "preAnalyzation;line=" + line );
				if( !is_old || (is_old && DateEx.dateFromTotalData( line ).largerThan( last_read_line_date )) ){
					is_old = false;
					char ch = line.charAt( 0 );
					if( ch == '1' ){
						analyzeTypeI( line, tantra.AnalyzeMode.PRE_ANALYZATION );
						final_line = line;
						tantra.settings.last_read_line = line;
					}else if( ch == '2' ){
						analyzeTypeII( line );
						tantra.settings.last_read_line = line;
					}else if( ch == '3' || ch == '4' || ch == '6' ){
						analyzeTypeIII( line );
						tantra.settings.last_read_line = line;
					}else if( ch == '5' ){
						analyzeTypeIII( line );
					}
				}else{
					if( line.equals( tantra.settings.last_read_line ) ){
						is_old = false;
					}
				}
			}
			sr.close();
		}catch( Exception ex ){
			tantra.printError( ex );
		}

		// 最新の週のweeklyリストを作成し，アップロードする
		if( !final_line.equals( "" ) ){
			String fname;
			DateEx week_start = DateEx.dateFromTotalData( final_line )
					.weekStart();
			fname = tantra.settings.target_kruma + "_"
					+ week_start.toString( Format.yyyyMMdd ) + ".html";
			Html.createRanking( week_start );
			tantra.uploader_main.registerUpload( tantra.settings.html_location
					+ File.separator + fname );
		}

	}


	/**
	 * 
	 */
	private static void uwscPreAnalyzation(){
		//System.out.println( "uwscPreAnalyzation" );
		int no_of_failure = 0;
		// LOCKモードへの移行を10回試みる
		while( !uwsc_server.enterLockMode() && no_of_failure < 10 ){
			// System.out.println( "no_of_failure=" + no_of_failure );
			no_of_failure += 1;
			System.out.println( "no_of_failure=" + no_of_failure );
			try{
				Thread.sleep( 1500 );
			}catch( InterruptedException e ){
				tantra.printError( e );
			}
		}
		// System.out.println( "uwscPreAnalyzation::entered lock mode" );
		uwsc_server.uwsc_current_record = tantra.settings.last_uwsc_log_entry;
		if( uwsc_server.openLogFile() ){
			UwscLogEntry entry;
			entry = uwsc_server.read();
			while( !entry.isNull() ){
				// System.out.println( "entry.entry_type=" + entry.entry_type );
				tantra.last_entry = entry;
				tantra.running_info.analyze( entry );
				entry = uwsc_server.read();
			}
			uwsc_server.closeLogFile();
		}
		uwsc_server.exitLockMode();
		// System.out.println( "uwscPreAnalyzation::exited lock mode" );
		tantra.running_info.createHtml();
		// System.out.println( "uwscPreAnalyzation::return" );
	}


	/**
	 * total dataの，行頭が1である行を処理します
	 * 
	 * @param line
	 *        解析する行データ。
	 * @param mode
	 *        解析するモード。
	 */
	private static void analyzeTypeI( String line, tantra.AnalyzeMode mode ){
		// System.out.println( "analyzeTypeI" );
		String token[];
		token = line.split( "," );
		String winner = token[1];
		String loser = token[2];
		// PersonalData pd_winner, pd_loser;
		DateEx line_date;
		// boolean exist_suspect = false;

		// htmlファイルの作成処理
		// System.out.println( "analyzeTypeI:line=" + line );
		line_date = DateEx.dateFromTotalData( line );
		// System.out.println( "analyzeTypeI:line_date=" +
		// line_date.getyyyy_MM_dd() + " " + line_date.gethh_mm_ss() );
		DateEx week_start = line_date.weekStart();
		// System.out.println( "analyzeTypeI:week_start=" +
		// week_start.getyyyy_MM_dd() + " " + week_start.gethh_mm_ss() );
		if( kruma_weekly_currentSunday.ymd() < week_start.ymd() ){
			// System.out.println( "analyzeTypeI;next week" );
			Html.createRanking( kruma_weekly_currentSunday );

			tantra.kruma_weekly.removeAllElements();
			int i;
			for( i = 0; i <= 6; i++ ){
				tantra.population.createPng( i );
			}

			tantra.population.createHtml();

			tantra.fixed.fixFiles( kruma_weekly_currentSunday );

			tantra.population.initialize( week_start );
			kruma_weekly_currentSunday = week_start;
		}

		int index_winner, index_loser;

		// 聖地人リストの更新
		index_winner = PersonalData.indexOf( tantra.kruma, winner, true );
		index_loser = PersonalData.indexOf( tantra.kruma, loser, true );
		PersonalData.manageLordValue( tantra.kruma.elementAt( index_winner ),
				tantra.kruma.elementAt( index_loser ) );
		tantra.kruma.elementAt( index_winner ).win += 1;
		tantra.kruma.elementAt( index_winner )
				.calculateActivityTime( line_date );
		tantra.kruma.elementAt( index_loser ).lose += 1;
		tantra.kruma.elementAt( index_loser ).calculateActivityTime( line_date );

		// ウィークリーランキングの更新
		index_winner = PersonalData.indexOf( tantra.kruma_weekly, winner, true );
		index_loser = PersonalData.indexOf( tantra.kruma_weekly, loser, true );
		PersonalData.manageLordValue( tantra.kruma_weekly
				.elementAt( index_winner ), tantra.kruma_weekly
				.elementAt( index_loser ) );
		tantra.kruma_weekly.elementAt( index_winner ).win += 1;
		tantra.kruma_weekly.elementAt( index_winner ).calculateActivityTime(
				line_date );
		tantra.kruma_weekly.elementAt( index_loser ).lose += 1;
		tantra.kruma_weekly.elementAt( index_loser ).calculateActivityTime(
				line_date );

		// 聖地人口の計算
		tantra.population.add( winner, line_date );
		tantra.population.add( loser, line_date );

		if( mode == tantra.AnalyzeMode.ANOTHER ){
			Html.createRanking( week_start );

			tantra.population.createPng( line_date.dayOfWeek() );
			/*
			 * tantra.uploader_main.registerUpload(
			 * tantra.settings.html_location + File.separator + "picture_" +
			 * tantra.settings.target_kruma + "_" +
			 * tantra.population.currentDate( line_date.dayOfWeek() ) .toString(
			 * Format.yyyyMMdd ) + ".png" );
			 */
			tantra.population.createHtml();
		}
	}


	/**
	 * total dataの，行頭が2である行を処理します
	 * 
	 * @param line
	 *        解析する行データ。
	 */
	private static void analyzeTypeII( String line ){
		// 2,S,A,{yyyyMMdd},{HH:mm:ss} => 主神 S の使徒 A がカーラを拾った（or登録した）
		String token[];
		token = line.split( "," );
		String lord = token[1];
		String person = token[2];
		int index_person = PersonalData.indexOf( tantra.kruma, person, true );
		PersonalData pd = tantra.kruma.elementAt( index_person );
		pd.brahma = false;
		pd.vishnu = false;
		pd.shiva = false;
		if( lord.equals( Lord.BRAHMA ) ){
			pd.brahma = true;
		}else if( lord.equals( Lord.VISHNU ) ){
			pd.vishnu = true;
		}else if( lord.equals( Lord.SHIVA ) ){
			pd.shiva = true;
		}

		// 活動時間の更新
		DateEx line_date = DateEx.dateFromTotalData( line );
		pd.calculateActivityTime( line_date );

		// 人口の計算
		tantra.population.add( person, line_date );

		// krumaを更新
		tantra.kruma.setElementAt( pd, index_person );

		// グラフを作成
		int i = line_date.dayOfWeek();
		tantra.population.createPng( i );
		/*
		 * tantra.uploader_main.registerUpload( tantra.settings.html_location +
		 * File.separator + "picture_" + tantra.settings.target_kruma + "_" +
		 * tantra.population.currentDate( i ).toString( Format.yyyyMMdd ) +
		 * ".png" );
		 */
	}


	/**
	 * total dataの，行頭が3,4または5である行を処理します
	 * 
	 * @param line
	 *        解析する行データ。
	 */
	private static void analyzeTypeIII( String line ){
		// 3,A,S,{yyyyMMdd},{HH:mm:ss} => A は主神 S の祝福を受けた
		// 4,A,S,{yyyyMMdd},{HH:mm:ss} => A が主神 S を選択した
		String token[];
		token = line.split( "," );
		String person = token[1];
		String lord = token[2];
		int index_person = PersonalData.indexOf( tantra.kruma, person, true );
		PersonalData pd = tantra.kruma.elementAt( index_person );
		pd.brahma = false;
		pd.vishnu = false;
		pd.shiva = false;
		if( lord.equals( Lord.BRAHMA ) ){
			pd.brahma = true;
		}else if( lord.equals( Lord.VISHNU ) ){
			pd.vishnu = true;
		}else if( lord.equals( Lord.SHIVA ) ){
			pd.shiva = true;
		}
		tantra.kruma.setElementAt( pd, index_person );
	}


	/**
	 * ロックファイルがなくなるまで待機する
	 */
	private static void waitEditing(){
		String respond = CURRENT_DIR + File.separator + "aye ma'am";
		try{
			FileOutputStream fos = new FileOutputStream( respond );
			fos.close();
			tantra.pushConsoleLine( "waiting" );
			Thread.sleep( 5000 );
			if( !tantra.exists( CURRENT_DIR + File.separator + "wait" ) ){
				if( tantra.exists( respond ) ){
					tantra.kill( respond );
				}
				tantra.pushConsoleLine( "exit wait mode" );
				return;
			}
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * 定時処理
	 */
	private static void timerTick(){
		emergencyExit();
		DateEx last_read_line_date = new DateEx();
		if( tantra.settings.last_read_line.length() > 0 ){
			last_read_line_date = DateEx.dateFromTotalData( tantra.settings.last_read_line );
		}
		try{
			// String fname;
			UwscLogEntry entry;
			if( uwsc_server.enterLockMode() && uwsc_server.openLogFile() ){
				// uwsc_is_locked = true;
				entry = uwsc_server.read();
				while( !entry.isNull() ){
					tantra.running_info.analyze( entry );
					tantra.last_entry = entry;
					entry = uwsc_server.read();
				}
				uwsc_server.closeLogFile();
				uwsc_server.exitLockMode();
			}
			tantra.running_info.createHtml();

			// chat log から新しいログデータを取り出し，解析
			String line, edited_line;
			line = log_server.nextLog();
			boolean added = false;
			while( line.length() > 0 ){
				edited_line = editLine( line );
				// System.out.println( "edited_line=" + edited_line );
				if( edited_line.length() > 0 ){
					current_date = DateEx.dateFromTotalData( edited_line );
					tantra.pushConsoleLine( edited_line );

					if( !is_old || 
							(is_old && current_date.largerThan( last_read_line_date ) ) ){
						is_old = false;
						tantra.settings.last_read_line = edited_line;
						last_read_line_date = current_date;
						TOTAL.write( edited_line );
						TOTAL.newLine();
						char ch = edited_line.charAt( 0 );
						if( ch == '1' ){
							analyzeTypeI( edited_line,
									tantra.AnalyzeMode.ANOTHER );
						}else if( ch == '2' ){
							analyzeTypeII( edited_line );
						}else if( ch == '3' || ch == '4' || ch == '5' ){
							analyzeTypeIII( edited_line );
						}
						added = true;						
					}else{
						if( tantra.settings.last_read_line.equals( edited_line ) ){
							is_old = false;
						}
					}

				}
				TOTAL.flush();
				tantra.settings.last_read_log = line;
				emergencyExit();
				line = log_server.nextLog();
				tantra.settings.last_uwsc_log_entry = uwsc_server.uwsc_current_record;
				tantra.settings.write( CURRENT_DIR + File.separator
						+ tantra.SETTING_FILE_NAME );
			}

			// total dataに何らかの行が加えられた場合。とりあえずファイルを全部作成。
			if( added ){
				Html.createRanking( kruma_weekly_currentSunday );
				tantra.population.createHtml();
				tantra.population.createPng( current_date.dayOfWeek() );
			}

			writeDump();
			Html.createIndex();
			int j;
			for( j = 0; j <= current_date.dayOfWeek(); j++ ){
				tantra.population.createPng( j );
			}

			printUnknownJobCharacter();

			DateEx tmpd = new DateEx();
			tmpd.now();
			if( tmpd.ymd() > tantra.settings.last_zipped_date.ymd() ){
				TOTAL.close();

				String fpath;
				fpath = tantra.settings.html_location + File.separator
						+ "tantra_data_" + tantra.settings.target_kruma + "_"
						+ tmpd.toString( Format.yyyyMMdd ) + ".zip";
				compress( fpath, tantra.settings.total_data_location
						+ File.separator + tantra.TOTAL_DATA_FILE_NAME );
				tantra.uploader_backup.registerUpload( fpath );

				fpath = tantra.settings.html_location + File.separator
						+ "job_list_" + tantra.settings.target_kruma + "_"
						+ tmpd.toString( Format.yyyyMMdd ) + ".zip";
				compress( fpath, tantra.settings.total_data_location
						+ File.separator + tantra.JOB_LIST );
				tantra.uploader_backup.registerUpload( fpath );

				tmpd = new DateEx( tmpd.year(), tmpd.month(), tmpd.day(), 0, 0,
						0 );
				tantra.settings.last_zipped_date = tmpd;
				tantra.settings.last_uwsc_log_entry = uwsc_server.uwsc_current_record;
				tantra.settings.write( CURRENT_DIR + File.separator
						+ tantra.SETTING_FILE_NAME );

				TOTAL = new BufferedWriter( new OutputStreamWriter(
						new FileOutputStream(
								tantra.settings.total_data_location
										+ File.separator
										+ tantra.TOTAL_DATA_FILE_NAME, true ),
						tantra.ShiftJIS ) );
			}

			// 
			tantra.uploader_main.tryUpload();
		}catch( Exception ex ){
			tantra.printError( ex );
		}
	}


	/**
	 * 種族が未判定のキャラクタをファイルにリストアップします。
	 */
	private static void printUnknownJobCharacter(){
		if( tantra.kruma.size() <= 0 ) return;
		int i;
		String fpath = tantra.settings.html_location + File.separator
				+ "unknown_job_" + tantra.settings.target_kruma + ".txt";
		try{
			BufferedWriter sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( fpath ), tantra.ShiftJIS ) );
			DateEx d = new DateEx();
			d.now();
			sw.write( "// generator : tantra_data version " + tantra.ver );
			sw.newLine();
			sw.write( "// date : " + d.toString( Format.yyyy_MM_dd ) + " "
					+ d.toString( Format.hh_mm_ss ) );
			sw.newLine();
			sw.write( "// target_kruma : " + tantra.settings.target_kruma );
			sw.newLine();
			sw.write( "//" );
			sw.newLine();
			sw.write( "// ご協力ありがとうございます．" );
			sw.newLine();
			sw.write( "// 名前の後の数字を，キャラクタのジョブにあわせて変更してください．" );
			sw.newLine();
			sw.write( "// ジョブと数字の対応関係は以下のとおりです．" );
			sw.newLine();
			sw.write( "//   アス・ラク   : " + Job.Asura_Rakshasa.getValue() );
			sw.newLine();
			sw.write( "//   ディバ・ガル : " + Job.Deva_Garuda.getValue() );
			sw.newLine();
			sw.write( "//   ナガ・キン   : " + Job.Naga_Kimnara.getValue() );
			sw.newLine();
			sw.write( "//   ヤク・ガン   : " + Job.Yaksa_Gandharva.getValue() );
			sw.newLine();
			sw.write( "// ファイルの送信先はpbnasrpa at yahoo.co.jpです．どうぞよろしくお願いします．" );
			sw.newLine();
			sw.write( "// [START]" );
			sw.newLine();
			PersonalData pd;
			i = 0;
			int j;
			for( j = 0; j <= tantra.kruma.size() - 1; j++ ){
				pd = tantra.kruma.elementAt( j );
				if( tantra.joblist.getJob( pd.name ) == Job.Unknown
						&& pd.win + pd.lose > 0 ){
					sw.write( pd.name + "," + Job.Unknown.getValue() );
					sw.newLine();
					i += 1;
				}
			}
			sw.write( "// [END]" );
			sw.newLine();
			sw.write( "// unknown job character(s) in kruma : " + i + " / "
					+ tantra.kruma.size() );
			sw.newLine();
			sw.close();
			tantra.uploader_main.registerUpload( fpath );
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * 与えられたファイルを圧縮し、Zipファイルに追加する
	 * 
	 * @param zip_path
	 *        追加するzipファイルへのパス
	 * @param file_name
	 *        zip_pathに追加するファイルへのパス
	 */
	private static void compress( String zip_path, String file_name ){
		File zipFile = new File( zip_path );
		File target[] = { new File( file_name ) };
		try{
			if( !tantra.exists( file_name ) ) return;
			ZipFile zf = new ZipFile();
			zf.zip( target, zipFile );
		}catch( Exception e ){
			tantra.printError( e );
		}
	}

	
	/**
	 * 
	 */
	private static void emergencyExit(){
		try{
			if( tantra.exists( CURRENT_DIR + File.separator + "terminate" ) ){
				if( TOTAL != null )	TOTAL.close();
				writeDump();
				tantra.settings.last_uwsc_log_entry = uwsc_server.uwsc_current_record;
				tantra.settings.write( CURRENT_DIR + File.separator
						+ tantra.SETTING_FILE_NAME );
				tantra.kill( CURRENT_DIR + File.separator + "terminate" );
				System.exit( 0 );
			}
		}catch( Exception e ){
			e.printStackTrace();
		}
	}

}
